/**
 * WebPM Principal 
 * mharris
 *
 * TODO:
 *
 */ 

package webpmsec;

import java.security.Principal;

/**
 * <p> This class implements the <code>Principal</code> interface
 * and represents a WebPM user.
 *
 * <p> Principals such as this <code>webpmPrincipal</code>
 * may be associated with a particular <code>Subject</code>
 * to augment that <code>Subject</code> with an additional
 * identity.  Refer to the <code>Subject</code> class for more information
 * on how to achieve this.  Authorization decisions can then be based upon 
 * the Principals associated with a <code>Subject</code>.
 * 
 * @version 1.4, 01/11/00
 * @see java.security.Principal
 * @see javax.security.auth.Subject
 */
public class webpmPrincipal implements java.security.Principal, java.io.Serializable {

    /**
     * @serial
     */
    private String name;

    /**
     * Create a webpmPrincipal with a default username.
     *
     * <p>
     *
     * @param name the default username for this user.
     *
     * @exception NullPointerException if the <code>name</code>
     *			is <code>null</code>.
     */
    public webpmPrincipal(String name) {
	if (name == null)
	    throw new NullPointerException("illegal null input");

	this.name = name;
    }

    /**
     * Return the default username for this <code>webpmPrincipal</code>.
     *
     * <p>
     *
     * @return the default username for this <code>webpmPrincipal</code>
     */
    public String getName() {
	return name;
    }

    /**
     * Return a string representation of this <code>webpmPrincipal</code>.
     *
     * <p>
     *
     * @return a string representation of this <code>webpmPrincipal</code>.
     */
    public String toString() {
	return("webpmPrincipal:  " + name);
    }

    /**
     * Compares the specified Object with this <code>webpmPrincipal</code>
     * for equality.  Returns true if the given object is also a
     * <code>webpmPrincipal</code> and the two webpmPrincipals
     * have the same username.
     *
     * <p>
     *
     * @param o Object to be compared for equality with this
     *		<code>webpmPrincipal</code>.
     *
     * @return true if the specified Object is equal equal to this
     *		<code>webpmPrincipal</code>.
     */
    public boolean equals(Object o) {
	if (o == null)
	    return false;

        if (this == o)
            return true;
 
        if (!(o instanceof webpmPrincipal))
            return false;
        webpmPrincipal that = (webpmPrincipal)o;

	if (this.getName().equals(that.getName()))
	    return true;
	return false;
    }
 
    /**
     * Return a hash code for this <code>webpmPrincipal</code>.
     *
     * <p>
     *
     * @return a hash code for this <code>webpmPrincipal</code>.
     */
    public int hashCode() {
	return name.hashCode();
    }
}

